Host

Struct Host 

pub struct Host(/* private fields */);
The driver will operate as a host

This means that - unless the chip has a native Thread suppoort - it needs to be connected via UART or SPI to another chip which does have native Thread support and which is configured to operate in RCP mode