pub struct SrpService<'a, SI, TI> {
pub name: &'a str,
pub instance_name: &'a str,
pub subtype_labels: SI,
pub txt_entries: TI,
pub port: u16,
pub priority: u16,
pub weight: u16,
pub lease_secs: u32,
pub key_lease_secs: u32,
}Expand description
An SRP service that can be registered with the OpenThread stack.
Fields
name: &'a strThe service name.
instance_name: &'a strThe instance name.
subtype_labels: SIThe subtype labels.
txt_entries: TIThe TXT entries.
port: u16The service port.
priority: u16The service priority.
weight: u16The service weight.
lease_secs: u32The service lease time in seconds.
Set to 0 to use the default value as specified in SrpConf.
key_lease_secs: u32The service key lease time in seconds.
Set to 0 to use the default value as specified in SrpConf.