pub type OutSrpService<'a> = SrpService<'a, OutSrpSubtypeLabelsIter<'a>, OutSrpTxtEntriesIter<'a>>;Expand description
Type alias for an SRP service as returned by
OpenThread::srp_services.
Aliased Type
pub struct OutSrpService<'a> {
pub name: &'a str,
pub instance_name: &'a str,
pub subtype_labels: OutSrpSubtypeLabelsIter<'a>,
pub txt_entries: OutSrpTxtEntriesIter<'a>,
pub port: u16,
pub priority: u16,
pub weight: u16,
pub lease_secs: u32,
pub key_lease_secs: u32,
}Fields
name: &'a strThe service name.
instance_name: &'a strThe instance name.
subtype_labels: OutSrpSubtypeLabelsIter<'a>The subtype labels.
txt_entries: OutSrpTxtEntriesIter<'a>The TXT entries.
port: u16The service port.
priority: u16The service priority.
weight: u16The service weight.
lease_secs: u32The service lease time in seconds.
Set to 0 to use the default value as specified in SrpConf.
key_lease_secs: u32The service key lease time in seconds.
Set to 0 to use the default value as specified in SrpConf.